/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvglcanvas.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVGLCANVAS_H
#define QVGLCANVAS_H

#include <QVWorker>
#include <QVMatrix>
#include <QVQuaternion>

#include <qvgui/qv3dmodel.h>

#ifndef DOXYGEN_IGNORE_THIS
/*!
@class QVGLCanvas qvglcanvas.h qvgui/qvglcanvas.h
@brief 3d scene representation widget.

This widget allows constructing a 3d scene and connecting the location matrices of the different objects to worker output properties, so it is useful for 3d representation of physical scenes, composed by different objects. An example usage follows:

\code
[...]

CameraCalibrator worker("Corners Worker");

[...]

QVGLCanvas glCanvas("3d camera location");
glCanvas.add(QV3DModel::referenceCoordinates(0.75, true), "Coordinate axis");
glCanvas.add(cameraModel(0.25, 0.5), "Camera");

glCanvas.linkModelMatrix(worker, "Extrinsic matrix" , "Camera");

[...]

\endcode

<i>Extrinsic matrix</i> should be a property included in the <i>worker</i> object, containing a @ref QVMatrix object. This matrix should represent a location matrix of dimensions 3x4, in the form <i>[R | t]</i>, where <i>R</i> is a rotation matrix, and <i>t</i> is a location vector. The model named <i>Camera</i>, added with the method @ref add, will move in the scene according to the values of the location matrix.

@todo Complete this documentation

@ingroup qvgui
*/
class QVGLCanvas : public QGLWidget, public QVPropertyContainer
{
Q_OBJECT

public:
	QVGLCanvas(const QString &title, QWidget* parent = 0);
	~QVGLCanvas ();

	virtual void init() {};
	virtual void display() {};
	virtual void reshape(int w, int h) {};

	void linkModelMatrix(QVWorker &worker, const QString &matrixPropertyName, const QString &objectName);
	void add(const QV3DModel &model, const QString &name);
	GLuint bindTexture(const QVImage<uChar,3> &image, GLenum target = GL_TEXTURE_2D);
	//GLuint bindTexture(const QImage &image, GLenum target = GL_TEXTURE_2D);
	//GLuint bindTexture(const QVImage<uChar>);

signals:
	void closed();

protected:

	void initializeGL();
	void paintGL();
	void resizeGL(int w, int h );

	void drawModels();

	void mousePressEvent(QMouseEvent * event);
	void mouseReleaseEvent(QMouseEvent *event);
	void mouseMoveEvent(QMouseEvent *event);
	void wheelEvent(QWheelEvent *event);
	void keyPressEvent(QKeyEvent *event);
	
	void closeEvent(QCloseEvent * event);

	int beginx, beginy;
	QVQuaternion trackballQuat;
	double zoom;
	bool pressedleft,pressedright;
	double maxwide;
	double cx,cy,cz;
};

#endif
#endif
